#!/bin/bash

### The simulation part of the neural network. This is run by individual members in parallel with each other
### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=%EXPID%/EXP00/
PROJ=pisces1d/ORCA2_OFF_PISCES/



### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob
numfiles=(${SCRHOME}${MODEL}parameters/*)
numfiles=${#numfiles[@]}

INPUTS=input/
XIOS=xios-2.5/
NEMO=release-4.0/

if [ $numfiles -eq $((1)) ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    cd $SCRHOME/%EXPID%/
    mkdir -p EXP00/%MEMBER%
    cd EXP00/%MEMBER%
    cp -R ${SCRHOME}${PROJ}${NEMO}cfgs/PISCES1D/EXP00/nemo .
    cp -R ${SCRHOME}${PROJ}${NEMO}cfgs/PISCES1D/EXP00/namelist* .
    cp -R ${SCRHOME}${PROJ}${NEMO}cfgs/PISCES1D/EXP00/*.xml .
    cp -R ${SCRHOME}${PROJ}${INPUTS}cropped/* .
fi



### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file

member=%MEMBER%
member=${member:2}

input=$SCRHOME${MODEL}/parameters/00.csv

count=0
while read -r line
do
    IFS=' ' read -r -a array <<< "$line"   # local machine - comma, mn - space
    printf -v ind "%02d\n" $count
    echo $ind $member
    if [ $ind == $member ]
    then
        xremip=${array[0]}
        wsbio=${array[1]}
        grazflux=${array[2]}
    fi
    count=$((count+1))
done < "$input"

echo $xremip
echo $wsbio
echo $grazflux

cd $SCRHOME${MODEL}%MEMBER%

sed -i "/xremip /c\   xremip    =  $xremip"  namelist_pisces_ref
sed -i "/wsbio /c\   wsbio     =  $wsbio" namelist_pisces_ref
sed -i "/grazflux /c\   grazflux  =  $grazflux" namelist_pisces_ref



### 3. Runs the simulation

cd $SCRHOME/%EXPID%/EXP00/%MEMBER%

module load intel/2017.4
module load impi/2017.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo



### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

REF=%EXPID%/EXP00/default/obs/
MODEL=%EXPID%/EXP00/%MEMBER%/
OBS=obs/

module load python
module load netcdf/4.4.1.1

function getcost {

cost="$(python - $1 $2 $3 << END

from netCDF4 import Dataset
import numpy as np
import os
import math

# get the reference bias
os.chdir('$1')
f=open('refbias.txt')
refbias=f.readlines()
refbias=float(refbias[0])
f.close()

# get the observed data
os.chdir('$2')
obs=Dataset('bbp700_pisces_climatology_lab1d.nc')
bbp700obs = np.array(obs.variables['bbp700_all_mean'])
bbp700obs = np.reshape(bbp700obs, (np.shape(bbp700obs)[0], np.shape(bbp700obs)[1]))
bbp700obs[bbp700obs == 0.0] = np.nan
obs.close()

# get the model data
os.chdir('$3')
mod = Dataset('PISCES1D_1d_19600101_19691231_ptrc_T.nc')
bbp700mod = 2500*(np.array(mod.variables['POC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY'])[-60:, :, 1, 1])
for i in range(12):
    bbp700mod[i] = np.mean(bbp700mod[i::12],0)
bbp700mod = bbp700mod[:12]
bbp700mod[bbp700mod == 200000039192547557376] = np.nan
mod.close()

# converting the data into logspace and normalising based on observed data
bbp700obs=np.log(bbp700obs)
bbp700mod=np.log(bbp700mod)

std=np.nanstd(bbp700obs)
mn=np.nanmean(bbp700obs)

bbp700obs=(bbp700obs-mn)/std
bbp700mod=(bbp700mod-mn)/std

# calculating the cost
bias=np.nanmean(bbp700obs-bbp700mod)
mbias=bias/refbias
mstd=np.nanstd(bbp700mod)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])
S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2)

ST=math.sqrt((mbias**2)+(S3**2))

print(ST)

END
)"
}

getcost $SCRHOME$REF $SCRHOME$PROJ$OBS $SCRHOME$MODEL

cost=(${cost})

echo $cost

cd $SCRHOME$MODEL
touch cost.txt
echo $cost > cost.txt